package gLite::VOMSLDAP;

=pod

=head1 NAME

gLite::VOMSLDAP

=head1 SYNOPSIS

Load the ldap synchronisation config file and return the parsed data
in a hash reference.

=head1 AUTHORS

Károly Lőrentey <Karoly.Lorentey@cern.ch>

Copyright (c) 2004 CERN, ELTE, on behalf of the EU DataGrid project.
For license conditions see LICENSE file or
L<http://www.edg.org/license.html>.

=cut

require 5.006;

use strict;
require Exporter;
my @ISA = qw(Exporter);
my @EXPORT_OK = qw(load_vo_config);

use FileHandle;
use XML::Parser;
use utf8;

our %config;
import gLite::Config;

my %vo;
my $vo;
my $map;

sub char_handler ($$) {
    my $p = shift;
    my $str = shift;
    my $parent = $p->current_element;

    # Skip whitespace-only strings.
    return if $str =~ /^\s+$/m;

    if (!defined $parent) {
	die "Character data found instead of root element\n";
    }
    elsif (grep ({$_ eq $parent;} qw(config vo ldap))) {
	die "Unexpected character data in <$parent> element\n";
    }
    elsif ($parent eq "host") {
	$vo->{host} = $str;
    }
    elsif ($parent eq "people") {
	$vo->{people} = $str;
    }
    elsif ($parent eq "map") {
	$vo->{maps}->{$map} = $str;
    }
}

sub start_handler {
    my $p = shift;
    my $e = shift;
    my %a = @_;

    my $parent = $p->current_element;

    if ($e eq "config") {
	die "<config> must be the root element\n" if defined $parent;
	%vo = ();
    }
    elsif ($e eq "vo") {
	die "<vo> must be directly under <config>\n" 
	    if !defined $parent or $parent ne "config";
	
	die "Required attributes are missing for <vo> element\n"
	    if grep {!exists $a{$_};} qw(name port);

	$vo{$a{name}} = \%a;
	$vo = $vo{$a{name}};
	$vo->{maps} = {};
    }
    elsif ($e eq "ldap") {
	die "<ldap> must be directly under a <vo> element\n"
	    if !defined $parent or $parent ne "vo";
    }
    elsif ($e eq "host") {
	die "<host> must be directly under a <ldap> element\n"
	    if !defined $parent or $parent ne "ldap";
    }
    elsif ($e eq "people") {
	die "<people> must be directly under a <ldap> element\n"
	    if !defined $parent or $parent ne "ldap";
    }
    elsif ($e eq "map") {
	die "<map> must be directly under a <ldap> element\n"
	    if !defined $parent or $parent ne "ldap";

	die "Required attribute 'dest' is missing in <map> element\n"
	    if !exists $a{dest};

	$map = $a{dest};
    }
    else {
	die "Unknown element: <$e>\n";
    }
}

sub end_handler ($$) {
    my $p = shift;
    my $e = shift;

    if ($e eq "vo") {
	die "No <host> found in <vo>\n" if !exists $vo->{host};
	die "No <people> found in <vo>\n" if !exists $vo->{people};
	die "No <map> found in <vo>\n" if !scalar keys %{$vo->{maps}};
    }
}

sub load_vo_config {
    my $ldap_config_file;
    $ldap_config_file = shift if (scalar @_) > 0;
    $ldap_config_file = $config{GLITE_LOCATION} .
	'/etc/voms-admin/ldap-vo-config.xml' 
	unless defined $ldap_config_file;
    $ldap_config_file = $config{GLITE_LOCATION_VAR} . 
	'/etc/voms-admin/ldap-vo-config.xml'
	unless -r $ldap_config_file;

    die "$ldap_config_file not found.\n" unless -r $ldap_config_file;

    my $p = new XML::Parser ();

    %vo = ();

    $p->setHandlers (Char => \&char_handler,
		     Start => \&start_handler,
		     End => \&end_handler);
    $p->parsefile ($ldap_config_file);
    
    return \%vo;
}

1;

# Please do not change this line.
# arch-tag: b268504f-69c0-4a14-9e1b-bcb4164c7d2a
